import os
import platform
import sys

from canonical.oops.wsgi import OopsWare
from django.conf import settings
from django.core.handlers.wsgi import WSGIHandler

# add path to sttings to system path if not already there
# check if path is already there to avoid path duplication when
# modwsgi is configured for code reloading
curdir = os.path.abspath(os.path.dirname(__file__))
if curdir not in sys.path:
    sys.path.append(curdir)

os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'

#
# add extra paths to system path if not already there
# check if path is already there to avoid path duplication when
# modwsgi is configured for code reloading
extra_paths = [path for path in settings.EXTRA_PYTHONPATH
               if path not in sys.path]
sys.path.extend(extra_paths)

os.environ['PGCONNECT_TIMEOUT'] = str(settings.PGCONNECT_TIMEOUT)

from identityprovider.wsgi import make_app

default_id = ''.join(x for x in platform.node() if x.isalpha())
appserver_id = getattr(settings, 'APPSERVER_ID', default_id)
appserver_id += settings.BRANDED_TEMPLATE_DIR

#Use make_app to enable the API, WSGIHandler to disable it:
if settings.API_ENABLED:
    app = make_app()
else:
    app = WSGIHandler()


def application(environ, start_response):
    oops_dir = getattr(settings, 'OOPS_DIR', '/tmp')
    serializer = 'canonical.oops.serializer.OOPSRFC822Serializer'
    oops = OopsWare(app, oops_dir=oops_dir, key=appserver_id, hide_meta=True,
        serializer_factory_name=serializer)
    return oops(environ, start_response)
