import os

from fabric.api import abort, settings

from .database import createdb, dropdb
from .django import get_django_settings, syncdb, manage
from .environment import bootstrap, virtualenv_local


def test(functional='false', coverage=False, extra=''):
    """Run unit (and optionally functional) tests."""
    cmd = ['python django_project/manage.py test --noinput', extra]

    # evaluate parameters
    functional = functional.lower() in ('t', 'true', '1')

    if functional:
        databases = get_django_settings('DATABASES')
        if databases['DATABASES']['default']['ENGINE'] == 'sqlite3':
            abort("You can't run functional tests on SQLite3")

        cmd.append("--testing_test_discovery_root=''")

    virtualenv_local(' '.join(cmd), capture=False)

def acceptance(headless='true', screenshot='false', report='',
        failfast='false', testcase=''):
    """Run acceptance tests only."""

    sst_host = os.getenv('SST_BASE_URL', None)
    if not sst_host or 'localhost' in sst_host:
        manage('create_test_team')

    headless = headless.lower() in ('t', 'true', '1')
    screenshot = screenshot.lower() in ('t', 'true', '1')
    failfast = failfast.lower() in ('t', 'true', '1')

    cmd = ['DJANGO_SETTINGS_MODULE=django_project.settings PYTHONPATH=.',
        'sst-run -m canonical.isd.tests.sst',
        '-d identityprovider/tests/acceptance']
    if headless:
        cmd.append('-x')
    if screenshot:
        cmd.append('-s')
    if report:
        cmd.append('-r %s' % report)
    if failfast:
        cmd.append('--failfast')
    if testcase:
        cmd.append(testcase)
    virtualenv_local(' '.join(cmd), capture=False)


def run(*args):
    """Run SSO using devserver."""
    virtualenv_local(
        'python django_project/manage.py runserver ' + ' '.join(args),
        capture=False)

def resetdb():
    """Drop and recreate then sync the database."""
    with settings(hide='warnings'):
        dropdb(warn_only=True)
    createdb()
    syncdb()

def jenkins():
    """Run the tests for jenkins."""
    bootstrap(jenkins=True)
    resetdb()
    # the functional tests depend on some data found in the fixtures
    manage('loaddata test')
    manage("jenkins --testing_test_discovery_root=''")
