# Copyright 2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
from __future__ import absolute_import
import os

from fabric.context_managers import cd

from .constants import APPS
from .environment import virtualenv_local


def get_django_settings(*keys):
    # setup the correct django environment
    os.environ['DJANGO_SETTINGS_MODULE'] = 'django_project.settings'
    # get django settings
    from django.conf import settings

    result = dict.fromkeys(keys)
    for key in keys:
        result[key] = getattr(settings, key)

    return result

def manage(command, *args):
    """Run manage.py command"""
    virtualenv_local("python django_project/manage.py {0} {1}".format(
        command, " ".join(args)), capture=False,)

def compilemessages(args=''):
    """Compile .po translation files into binary (.mo)."""
    cmd = 'python ../django_project/manage.py compilemessages'
    if args:
        cmd += " %s" % args
    for app in APPS:
        with cd(app):
            virtualenv_local(cmd, capture=False)

def makemessages():
    """Create/Update translation strings in .po files."""
    django_settings = get_django_settings('SUPPORTED_LANGUAGES')
    supported_languages = django_settings['SUPPORTED_LANGUAGES']
    for app in APPS:
        for lang in supported_languages:
            with cd(app):
                cmd = ("python ../django_project/manage.py makemessages "
                    "-l {0} -e .html,.txt")
                if '-' in lang:
                    language, country = lang.split('-')
                    locale_name = "{0}_{1}".format(language, country.upper())
                else:
                    locale_name = lang
                virtualenv_local(cmd.format(locale_name), capture=False)

def loaddata():
    pass

def syncdb():
    """Sync the database."""
    args = '--noinput'
    virtualenv_local("python django_project/manage.py syncdb %s" % args,
        capture=False)
