# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from django.conf.urls.defaults import patterns, url
from django.views.decorators.csrf import csrf_exempt

from piston.authentication import HttpBasicAuthentication
from piston.resource import Resource

from identityprovider.api10.handlers import (
    AccountsHandler,
    AuthenticationHandler,
    CaptchaHandler,
    RegistrationHandler,
    RootHandler,
)
from identityprovider.auth import (
    basic_authenticate,
    SSOOAuthAuthentication,
)


root_resource = Resource(handler=RootHandler)
captcha_resource = csrf_exempt(Resource(handler=CaptchaHandler))
registration_resource = csrf_exempt(Resource(handler=RegistrationHandler))
authentication_resource = csrf_exempt(Resource(handler=AuthenticationHandler,
    authentication=HttpBasicAuthentication(auth_func=basic_authenticate)))
accounts_resource = csrf_exempt(Resource(handler=AccountsHandler,
    authentication=SSOOAuthAuthentication()))

urlpatterns = patterns('',
    url(r'^$', root_resource,
        kwargs={'emitter_format': 'lazr.restful'}),
    url(r'^captchas$', captcha_resource,
        kwargs={'emitter_format': 'lazr.restful'}),
    url(r'^registration$', registration_resource,
        kwargs={'emitter_format': 'lazr.restful'}),
    url(r'^authentications$', authentication_resource,
        kwargs={'emitter_format': 'lazr.restful'}),
    url(r'^accounts$', accounts_resource,
        kwargs={'emitter_format': 'lazr.restful'}),
)

