from django.core.management.base import BaseCommand
from django.conf import settings

from identityprovider.models import EmailAddress, Person
from identityprovider.models.openidmodels import OpenIDRPConfig
from identityprovider.tests.utils import add_user_to_team
from identityprovider.tests import factory


class Command(BaseCommand):

    help = ("Create the test user, with team membership, required for "
            "acceptance tests.")

    def handle(self, *args, **options):
        email = settings.QA_ACCOUNT_EMAIL
        password = settings.QA_ACCOUNT_PASSWORD

        # hardcoded in tests
        name = 'isdtest'
        fullname = 'ISD Test'
        team = 'canonical-isd-hackers'
        allowed_sreg = ','.join([
            'fullname', 'nickname', 'email', 'timezone',
            'language'
        ])

        try:
            Person.objects.get(name=name)
        except Person.DoesNotExist:
            print 'Creating test team and user account.'
            f = factory.SSOObjectFactory()
            account = f.make_account(fullname, name, email, password)
            account.preferredemail = EmailAddress.objects.get(email=email)
            account.save()
            add_user_to_team(account, team, name)

            OpenIDRPConfig.objects.create(
                trust_root='http://localhost:8000/consumer',
                displayname='test',
                auto_authorize=False,
                allowed_sreg=allowed_sreg
            )
        else:
            print 'Test user and team already exists.'
