# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import datetime

from django.db import models

from identityprovider.models import Account, Person
from identityprovider.models.const import EmailStatus
from django.utils.translation import ugettext_lazy as _

__all__ = (
    'EmailAddress',
)


class EmailAddress(models.Model):
    email = models.TextField()
    lp_person = models.ForeignKey(Person, db_column='person',
        blank=True, null=True, editable=False)
    status = models.IntegerField(choices=EmailStatus._get_choices())
    date_created = models.DateTimeField(default=datetime.datetime.utcnow,
        blank=True, editable=False)
    account = models.ForeignKey(Account, db_column='account',
        blank=True, null=True)

    class Meta:
        app_label = 'identityprovider'
        db_table = u'emailaddress'
        verbose_name_plural = _('Email addresses')

    def __unicode__(self):
        return self.email

    def is_preferred(self):
        return self.status == EmailStatus.PREFERRED

    @property
    def person(self):
        if self.account is not None:
            person = self.account.person
        else:
            person = self.lp_person
        return person

    def account_admin_link(self):
        if self.account is None:
            return "None"
        return '<a href="../account/%s/">%s</a>' % (self.account.id,
            self.account.displayname)
    account_admin_link.allow_tags = True
