# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from django.db import models

__all__ = (
    'Continent',
    'Country',
    'Language',
    'LibraryFileContent',
    'LibraryFileAlias',
)


class Continent(models.Model):
    code = models.TextField(unique=True)
    name = models.TextField(unique=True)

    class Meta:
        app_label = 'identityprovider'
        db_table = u'continent'

    def __unicode__(self):
        return self.name


class Country(models.Model):
    iso3166code2 = models.CharField(unique=True, max_length=2)
    iso3166code3 = models.CharField(unique=True, max_length=3)
    name = models.TextField(unique=True)
    title = models.TextField(null=True, blank=True)
    description = models.TextField(null=True, blank=True)
    continent = models.ForeignKey(Continent, db_column='continent')

    class Meta:
        app_label = 'identityprovider'
        db_table = u'country'

    def __unicode__(self):
        return self.name


class Language(models.Model):
    code = models.TextField(unique=True)
    englishname = models.TextField(null=True, blank=True)
    nativename = models.TextField(null=True, blank=True)
    pluralforms = models.IntegerField(null=True, blank=True)
    pluralexpression = models.TextField(null=True, blank=True)
    visible = models.BooleanField()
    direction = models.IntegerField()
    uuid = models.TextField(null=True, blank=True)

    class Meta:
        app_label = 'identityprovider'
        db_table = u'language'

    def __unicode__(self):
        return self.englishname


class LibraryFileContent(models.Model):
    datecreated = models.DateTimeField(auto_now_add=True)
    datemirrored = models.DateTimeField(null=True, blank=True)
    filesize = models.IntegerField()
    sha1 = models.CharField(max_length=40)
    deleted = models.BooleanField(default=False)
    md5 = models.CharField(max_length=32)
    sha256 = models.CharField(max_length=64, null=True, blank=True)

    class Meta:
        app_label = 'identityprovider'
        db_table = u'libraryfilecontent'


class LibraryFileAlias(models.Model):
    content = models.ForeignKey(LibraryFileContent, db_column='content')
    filename = models.TextField()
    mimetype = models.TextField()
    expires = models.DateTimeField(null=True, blank=True)
    last_accessed = models.DateTimeField(auto_now=True)
    date_created = models.DateTimeField(auto_now_add=True)
    restricted = models.BooleanField(default=False)
    hits = models.IntegerField(default=0)

    class Meta:
        app_label = 'identityprovider'
        db_table = u'libraryfilealias'

    def __unicode__(self):
        return self.filename
