#6) [/new_account 5)] - Create a new account but do not use
#       this account to Login to Launchpad, you now have a personless account.
#       On Staging request private teams with a personless account and checks a private
#           LP resource
#       On Production go to the following sites with the personless account:
#       - http://shop.canonical.com
#       - http://wiki.canonical.com
#       - http://pastebin.canonical.com
#       - Also checks a private LP resource
#       The wiki & pastebin sites shouldn't allow access to the personless account.
#       Shop should allow login.


from canonical.isd.tests.sst import mail, config
from sst.actions import *

from identityprovider.tests.acceptance import _helpers as helpers

config.set_base_url_from_env()

# Some external sites have extraordinary wait times
set_wait_timeout(20)

email_address = mail.make_unique_test_email_address()
account_password = 'Admin007'

go_to('/+new_account')
assert_title('Create account')

write_textfield('id_displayname', "My Name")
write_textfield('id_email', email_address)
write_textfield('id_password', account_password)
write_textfield('id_passwordconfirm', account_password)
# Even though the recaptcha field is ignored for our tests, we do
# want to verify that it is on the page.
write_textfield('recaptcha_response_field', 'ignored')
click_button(get_element(name='continue'))

assert_title_contains('Registration mail sent')
assert_text_contains(get_element(id='content'),
    r'just emailed .* \(from .*\) to confirm your address\.', regex=True)

vcode = helpers.get_verification_code_for_address(email_address)

write_textfield(get_element(name='confirmation_code'), vcode)
click_button(get_element(css_class='btn'))

# We know that the confirmation code worked if we are now looking at the
# logged-in details page for the user.
wait_for(assert_title, "My Name's details")

# Depending whether we're on Production or Staging different tests
if get_base_url() == 'https://login.ubuntu.com/':  #Production
    # shop
    go_to('http://shop.canonical.com')
    wait_for(assert_title, 'Canonical Store')
    click_link(get_element(href="https://shop.canonical.com/login.php"))
    wait_for(get_element, id='id_email')
    write_textfield('id_email', email_address)
    write_textfield('id_password', account_password)
    click_button(get_element(name='continue'))
    click_button(get_element(name='yes'))
    wait_for(get_element, name='email_address')
    assert_element(value=email_address)

    # launchpad
    go_to('https://launchpad.net/~canonical-isd-hackers/+archive/internal-qa/+index')
    click_button(get_element(name='yes'))
    assert_element(tag='h1', text='Not allowed here')

    # wiki
    go_to('http://wiki.canonical.com')
    click_link(get_element(id="login"))
    go_to('https://wiki.canonical.com/CDO/ISD/Developers/StandupNotes')
    assert_element(tag='strong', text='You are not allowed to view this page.')

    # pastebin
    go_to('https://pastebin.canonical.com/')
    assert_element(tag='h1', text='You do not currently have access to the pastebin.')


else:  # Staging first use test consumer to request private teams, then go to staging LP
    go_to('/consumer')
    wait_for(assert_title, 'Django OpenID Example Consumer')
    toggle_checkbox('id_teams')
    write_textfield('id_request_teams', 'canonical-isd-hackers,canonical-isd,canonical')
    click_button(get_element(value='Begin'))
    wait_for(get_element, name='yes')
    click_button(get_element(name='yes'))
    wait_for(assert_title, 'Django OpenID Example Consumer')
    assert_text_contains(get_element(tag='div',css_class='message'),'OpenID authentication succeeded')
    assert_text_contains(get_element(tag='div',css_class='message'),'no teams data')

    helpers.skip_local()
    # staging launchpad, can't run this part locally
    go_to('https://staging.launchpad.net/~canonical-isd-hackers/+archive/internal-qa/+index')
    wait_for(get_element, id='id_email')
    write_textfield('id_email', email_address)
    write_textfield('id_password', account_password)
    click_button(get_element(name='continue'))
    wait_for(get_element, name='yes')
    click_button(get_element(name='yes'))
    assert_element(tag='h1', text='Not allowed here')
