from openid.consumer.consumer import Consumer
from openid.store.memstore import MemoryStore
from openid.consumer.discover import OPENID_2_0_TYPE as PROTOCOL_URI

from identityprovider.const import LAUNCHPAD_TEAMS_NS

from identityprovider.tests.functional.openidhelpers import complete_from_browser, make_endpoint
from identityprovider.tests.functional.helpers import FunctionalTestCase


class OpenIDTeamsTestCase(FunctionalTestCase):

    def test(self):
        # = Launchpad OpenID Teams Extension =

        # The Launchpad OpenID server implements a custom team membership
        # extension.  This allows a relying party to query whether the user is a
        # member of one or more teams.

        # First we'll set up the OpenID consumer:
        openid_store = MemoryStore()
        consumer = Consumer(session={}, store=openid_store)

        # Now perform an OpenID authentication request, querying membership in
        # four team names:
        #  * one that the user is a member of
        #  * one that does not exist
        #  * one that does exist but the user is not a member of
        #  * one that is actually the user's name

        endpoint = make_endpoint(
            PROTOCOL_URI, 'http://openid.launchpad.dev/+id/mark_oid')
        request = consumer.beginWithoutDiscovery(endpoint)


        request.message.namespaces.addAlias(LAUNCHPAD_TEAMS_NS, 'lp')
        request.addExtensionArg(
            LAUNCHPAD_TEAMS_NS, 'query_membership',
            'ubuntu-team,no-such-team,launchpad-beta-testers,mark')

        self.browser.open(request.redirectURL(
            'http://launchpad.dev/', 'http://launchpad.dev/+openid-consumer'))
        self.browser.getControl(name='email').value = 'mark@example.com'
        self.browser.getControl(name='password').value = 'test'
        self.browser.getControl(name='continue').click()
        # authorize sending team membership
        self.browser.getControl(name='ubuntu-team').value = True
        self.browser.getControl(name='yes', index=0).click()

        self.assertUrl('http://launchpad.dev/\+openid-consumer\?.*?')

        info = complete_from_browser(consumer, self.browser)

        self.assertEquals(info.status, 'success')
        self.assertEquals(info.getSigned(LAUNCHPAD_TEAMS_NS, 'is_member'),
                          'ubuntu-team')

        # The response reveals that the user is a member of the ubuntu-team.  As
        # specified, there is no difference in the response for non-existent
        # teams and teams that the user is not a member of.
