from openid.consumer.consumer import Consumer
from openid.store.memstore import MemoryStore
from openid.consumer.discover import OPENID_2_0_TYPE as PROTOCOL_URI

from identityprovider.const import LAUNCHPAD_TEAMS_NS
from identityprovider.models import OpenIDRPConfig

from identityprovider.tests.functional.openidhelpers import complete_from_browser, make_endpoint
from identityprovider.tests.functional.helpers import FunctionalTestCase


class OpenIDTeamsAutoAuthorizeTestCase(FunctionalTestCase):

    def test(self):
        # = Interaction of Launchpad OpenID Teams with Auto-Authorize =
        # Check that teams work well when requested by an auto-authorized RP.
        OpenIDRPConfig.objects.all().delete()
        rpconfig = self.create_openid_rp_config(
            trust_root='http://launchpad.dev/',
            displayname='Test RP', description='A test RP',
            auto_authorize=True, can_query_any_team=True)

        # First we'll set up the OpenID consumer:
        openid_store = MemoryStore()
        consumer = Consumer(session={}, store=openid_store)

        # Now perform an OpenID authentication request, querying membership in
        # four team names:
        #  * one that the user is a member of and is public
        #  * one that the user is a member of, but is private
        #  * one that does not exist
        #  * one that does exist but the user is not a member of

        endpoint = make_endpoint(
            PROTOCOL_URI, 'http://openid.launchpad.dev/+id/cCGE3LA')
        request = consumer.beginWithoutDiscovery(endpoint)

        request.message.namespaces.addAlias(LAUNCHPAD_TEAMS_NS, 'lp')
        request.addExtensionArg(
            LAUNCHPAD_TEAMS_NS, 'query_membership',
            'ubuntu-team,no-such-team,launchpad-beta-testers,myteam')

        self.browser.open(request.redirectURL(
            'http://launchpad.dev/', 'http://launchpad.dev/+openid-consumer'))
        self.browser.getControl(name='email').value = 'member@canonical.com'
        self.browser.getControl(name='password').value = 'test'
        self.browser.getControl(name='continue').click()

        self.assertUrl('http://launchpad.dev/\+openid-consumer\?.*?')

        info = complete_from_browser(consumer, self.browser)

        self.assertEquals(info.status, 'success')
        self.assertEquals(info.getSigned(LAUNCHPAD_TEAMS_NS, 'is_member'),
                          'ubuntu-team,myteam')

        # The response reveals that the user is a member of the ubuntu-team,
        # and myteam as it's allowed to query private teams too.
