from openid.consumer.consumer import Consumer
from openid.store.memstore import MemoryStore
from openid.consumer.discover import OPENID_2_0_TYPE as PROTOCOL_URI

from identityprovider.tests.functional.openidhelpers import (complete_from_browser,
                              make_identifier_select_endpoint)
from identityprovider.tests.functional.helpers import FunctionalTestCase


class SSOAutoAuthorizeTestCase(FunctionalTestCase):

    def test(self):
        # = Single-Signon Workflow: Automatic Authorization =

        # For sites that are intended to look like an integrated part of
        # Canonical's single sign on system, we would like to avoid actively
        # asking the user to authenticate to the site.

        # As the user is likely to consider the sites to be part of a single
        # larger system, it just causes confusion.

        # First we will set up the helper view that lets us test the final
        # portion of the authentication process:

        # Next we'll set up the trust root we're using to have its requests
        # automatically authorized:

        self.create_openid_rp_config(
            trust_root='http://launchpad.dev/',
            displayname='Test RP', description='A test RP',
            auto_authorize=True)

        # If we are not logged in, automatically authorized sites act the same
        # as normal ones, and the user is presented with the login page:

        openid_store = MemoryStore()
        consumer = Consumer(session={}, store=openid_store)
        request = consumer.beginWithoutDiscovery(
            make_identifier_select_endpoint(PROTOCOL_URI))

        self.browser.open(request.redirectURL(
            'http://launchpad.dev/', 'http://launchpad.dev/+openid-consumer'))

        self.assertEquals(self.browser.title.strip(), "Log in")

        # When the user logs in, he will be directed back to the relying party without
        # requesting for authorization as the trust root has automatic authorization
        # enabled.

        self.browser.getControl(name='email').value = 'mark@example.com'
        self.browser.getControl(name='password').value = 'test'
        self.browser.getControl(name='continue').click()

        self.assertUrl('http://launchpad.dev/\+openid-consumer\?.*?')

        info = complete_from_browser(
            consumer, self.browser, 'http://openid.launchpad.dev/+id/mark_oid')

        self.assertEquals(info.status, 'success')
        self.assertEquals(info.endpoint.claimed_id,
                          'http://openid.launchpad.dev/+id/mark_oid')

        # If the user is already logged in, he will be directed back to the
        # relying party immediately.

        request = consumer.beginWithoutDiscovery(
            make_identifier_select_endpoint(PROTOCOL_URI))
        self.browser.open(request.redirectURL(
            'http://launchpad.dev/', 'http://launchpad.dev/+openid-consumer'))

        self.assertUrl('http://launchpad.dev/\+openid-consumer\?.*?')

        info = complete_from_browser(
             consumer, self.browser, 'http://openid.launchpad.dev/+id/mark_oid')

        self.assertEquals(info.status, 'success')
        self.assertEquals(info.endpoint.claimed_id,
                          'http://openid.launchpad.dev/+id/mark_oid')
