from openid.consumer.consumer import Consumer
from openid.store.memstore import MemoryStore
from openid.consumer.discover import OPENID_2_0_TYPE as PROTOCOL_URI

from identityprovider.utils import encrypt_launchpad_password
from identityprovider.models.account import Account

from identityprovider.tests.functional.openidhelpers import complete_from_browser, make_identifier_select_endpoint
from identityprovider.tests.functional.helpers import FunctionalTestCase


class SSOWorkflowSwitchUserTestCase(FunctionalTestCase):

    def test(self):
        # = Launchpad Single-Signon Workflow: Switching Users =

        # If a user is making use of a public or shared computer, and wants to
        # log into a Launchpad-SSO website, they may find that someone else has
        # logged into Launchpad.  In this case, the user can log out and then
        # back in as part of the authentication process.

        # First we will set up the helper view that lets us test the final
        # portion of the authentication process:

        # We will first authenticate as Mark Shuttleworth:
        openid_store = MemoryStore()
        consumer = Consumer(session={}, store=openid_store)

        request = consumer.beginWithoutDiscovery(
            make_identifier_select_endpoint(PROTOCOL_URI))
        self.browser.open(request.redirectURL(
            'http://launchpad.dev/', 'http://launchpad.dev/+openid-consumer'))

        self.browser.getControl(name='email').value = 'mark@example.com'
        self.browser.getControl(name='password').value = 'test'
        self.browser.getControl(name='continue').click()
        self.browser.getControl(name='yes').click()

        self.assertUrl('http://launchpad.dev/\+openid-consumer\?.*?')

        # Now lets imagine that test@canonical.com starts using the computer and
        # wishes to log into a Launchpad-SSO web site.  They will be asked if
        # they want to authenticate as Mark Shuttleworth:

        request = consumer.beginWithoutDiscovery(
            make_identifier_select_endpoint(PROTOCOL_URI))
        self.browser.open(request.redirectURL(
            'http://launchpad.dev/', 'http://launchpad.dev/+openid-consumer'))

        self.assertEquals(self.browser.title.strip(),
                          "Authenticate to http://launchpad.dev/")

        # At this point, the user says that they are not Mark Shuttleworth,
        # which presents them with a login page:

        self.browser.getLink(id='logout-link').click()

        self.assertEquals(self.browser.title.strip(),
                          "You have been logged out")

        # They can now log in as test@canonical.com, and the appropriate
        # identity URL will be sent to the relying party:

        # First, we set the password.

        account = Account.objects.get_by_email('test@canonical.com')
        account.accountpassword.password = encrypt_launchpad_password('test')
        account.accountpassword.save()

        # Then we run the test
        self.browser.getLink(id="login-link").click()
        self.browser.getControl(name='email').value = 'test@canonical.com'
        self.browser.getControl(name='password').value = 'test'
        self.browser.getControl(name='continue').click()
        self.browser.getControl(name='yes').click()

        self.assertUrl('http://launchpad.dev/\+openid-consumer\?.*?')

        info = complete_from_browser(
            consumer, self.browser, 'http://openid.launchpad.dev/+id/name12_oid')

        self.assertEquals(info.status, 'success')
        self.assertEquals(info.endpoint.claimed_id,
                          'http://openid.launchpad.dev/+id/name12_oid')
