from openid.consumer.consumer import Consumer
from openid.store.memstore import MemoryStore
from openid.consumer.discover import OPENID_2_0_TYPE as PROTOCOL_URI

from identityprovider.tests.functional.openidhelpers import make_identifier_select_endpoint
from identityprovider.tests.functional.helpers import FunctionalTestCase


class SwitchUserTwiceTestCase(FunctionalTestCase):

    def test(self):
        # = Switching Users Twice =

        # If a user is already logged in when authenticating via OpenID, they
        # have the option to sign in as someone else.  This logs them out and
        # takes them to the login form.

        # By using the back button in their browser, it is possible for the user
        # to perform this action twice.  The result should be the same: the user
        # is taken to the login form.

        # To test this, we will first sign the user in.  First we'll set up the
        # OpenID consumer:
        openid_store = MemoryStore()
        consumer = Consumer(session={}, store=openid_store)

        # Then log in as Sample Person:

        request = consumer.beginWithoutDiscovery(
            make_identifier_select_endpoint(PROTOCOL_URI))
        self.browser.open(request.redirectURL(
            'http://launchpad.dev/', 'http://launchpad.dev/+openid-consumer'))

        self.assertEquals(self.browser.title.strip(), "Log in")

        self.browser.getControl(name='email').value = 'test@canonical.com'
        self.browser.getControl(name='password').value = 'test'
        self.browser.getControl(name='continue').click()
        self.browser.getControl(name='yes').click()

        self.assertUrl('http://launchpad.dev/\+openid-consumer\?.*?')

        # Now lets start a second authentication request, which should take us
        # to the authentication page asking if we want to sign in as someone
        # else:

        request = consumer.beginWithoutDiscovery(
            make_identifier_select_endpoint(PROTOCOL_URI))
        self.browser.open(request.redirectURL(
            'http://launchpad.dev/', 'http://launchpad.dev/+openid-consumer'))

        self.assertEquals(self.browser.title.strip(),
                          "Authenticate to http://launchpad.dev/")

        # Now log out, by saying that we are someone else:

        self.browser.getLink(id='logout-link').click()

        self.assertEquals(self.browser.title.strip(),
                          "You have been logged out")
