from urllib import urlencode

from identityprovider.tests.functional.helpers import FunctionalTestCase


class DirectedIdentityTestCase(FunctionalTestCase):

    def test(self):
        self.create_openid_rp_config(auto_authorize=False)

        # Our OpenID 1.1 implementation supports the OpenID 2.0 feature where
        # the consumer need not specify the identity URL, instead leaving it up
        # to the OpenID provider to determine it. We call this mode 'OP
        # Identifier' for lack of a better name.
        #
        # To do this, the consumer simply needs to specify the identity as
        # 'http://specs.openid.net/auth/2.0/identifier_select'.  Starting with
        # a new browser context, we are presented with a login screen.  After
        # authenticating to Launchpad, we return to the consumer:
        #
        # If authorization is successful, the consumer can determine the actual
        # identity from the response and do further validation as normal.

        args = urlencode({
            'openid.mode': 'checkid_setup',
            'openid.identity':
                'http://specs.openid.net/auth/2.0/identifier_select',
            'openid.return_to': 'http://launchpad.dev/+openid-consumer',
            'openid.trust_root': 'http://launchpad.dev/+openid-consumer',
            })
        setup_url = 'http://openid.launchpad.dev/+openid?%s' % args
        self.browser.open(setup_url)
        self.assertEquals(self.browser.title.strip(), "Log in")

        self.browser.getControl(name='email').value = 'mark@example.com'
        self.browser.getControl(name='password').value = 'test'
        self.browser.getControl(name='continue').click()

        self.assertUrl('http://openid.launchpad.dev/.*?/\+decide')

        self.browser.getControl(name='yes').click()

        self.assertContains('Consumer received GET')
        self.assertContains(
            'openid.identity:http://openid.launchpad.dev/+id/mark_oid')
        self.assertContains('openid.mode:id_res')
        self.assertContains(
            'openid.return_to:http://launchpad.dev/+openid-consumer')

        # If we happened to be logged into Launchpad previously, we would be
        # presented with an authorization screen instead:

        self.browser.open(setup_url)
        self.assertEquals(self.browser.title.strip(),
                          'Authenticate to http://launchpad.dev/+openid-consumer')

        self.browser.getControl(name='yes', index=0).click()

        self.assertUrl('http://launchpad.dev/\+openid-consumer\?.*?')
        self.assertContains('Consumer received GET')
        self.assertContains(
            'openid.identity:http://openid.launchpad.dev/+id/mark_oid')
        self.assertContains('openid.mode:id_res')
        self.assertContains(
            'openid.return_to:http://launchpad.dev/+openid-consumer')

