from urllib2 import HTTPError
from openid.yadis.discover import DiscoveryFailure

from identityprovider.tests.functional.helpers import FunctionalTestCase
from identityprovider.tests.functional.openidhelpers import get_endpoints


class DiscoveryTestCase(FunctionalTestCase):

    def test(self):
        # = OpenID Discovery =
        #
        # SSO provides a number of pages that can be used as OpenID URLs.
        # The OpenID discovery process can be run on them to discover the
        # endpoints.

        # == Persistent User Identity URLs ==
        # SSO assigns a persistent identifier to each user:
        endpoints = get_endpoints('http://openid.launchpad.dev/+id/mark_oid')

        self.assertEquals(endpoints, {
            'claimed_id': 'http://openid.launchpad.dev/+id/mark_oid',
            'endpoints': [{
                'local_id': 'http://openid.launchpad.dev/+id/mark_oid',
                'server_url': 'http://openid.launchpad.dev/+openid',
                'supports': ['http://specs.openid.net/auth/2.0/signon']
            }, {
                'local_id': 'http://openid.launchpad.dev/+id/mark_oid',
                'server_url': 'http://openid.launchpad.dev/+openid',
                'supports': ['http://openid.net/signon/1.1']
            }, {
                'local_id': 'http://openid.launchpad.dev/+id/mark_oid',
                'server_url': 'http://openid.launchpad.dev/+openid',
                'supports': ['http://openid.net/signon/1.0']
            }]
        })
        # # If the user enters a non-canonical form of the identifier, it is
        # # normalised to the standard form:
        # endpoints = get_endpoints(
        #     'http://openid.launchpad.dev/+id/mark_oid?foo=bar')

        # self.assertEquals(endpoints['claimed_id'],
        #                   'http://openid.launchpad.dev/+id/mark_oid')

        # == OP Identifier Discovery ==
        #
        # It is also possible to run discovery on the root directory as an OP
        # identifier.  This allows any Launchpad user to log in by entering this
        # URL.
        endpoints = get_endpoints('http://openid.launchpad.dev/')

        self.assertEquals(endpoints, {
            'claimed_id': 'http://openid.launchpad.dev/',
            'endpoints': [{
                'local_id': None,
                'server_url': 'http://openid.launchpad.dev/+openid',
                'supports': [
                    'http://specs.openid.net/auth/2.0/server',
                    'http://openid.net/extensions/sreg/1.1',
                    'http://ns.launchpad.net/2007/openid-teams',
                    ],
        }],})


        # == Invalid Persistent Identifiers ==
        #
        # Discovery is not possible on non-existent persistent identifiers:
        self.assertRaises(HTTPError,
                          self.browser.open,
                          'http://openid.launchpad.dev/+id/no-such-id')

        self.assertRaises(DiscoveryFailure,
                          get_endpoints,
                          'http://openid.launchpad.dev/+id/no-such-id')


        # == No OpenID for Invalid Users ==
        #
        # People who are not valid Launchpad users do not have OpenID
        # Identifiers:

        endpoints = get_endpoints('http://launchpad.dev/~matsubara')

        self.assertEquals(endpoints, {
            'claimed_id': 'http://launchpad.dev/~matsubara',
            'endpoints': []
        })

