from urllib2 import HTTPError

from django.core.cache import cache

from identityprovider.tests.functional.helpers import FunctionalTestCase


class RateLimitTestCase(FunctionalTestCase):

    def test(self):
        # First, clear everything from cache
        cache.clear()
        # = Rate limits =

        # Rate limiting is enforced only on POST requests to the login screen, on
        # a per-username, per-IP basis.
        # There are two settings that control the rate at which you can submit
        # requests: no more than LOGIN_LIMIT_REQUESTS requests can be submitted every
        # LOGIN_LIMIT_MINUTES minutes.
        # 
        # So, first, let's find out how many requests is our limit

        from django.conf import settings
        limit = getattr(settings, 'LOGIN_LIMIT_REQUESTS', 20)
        # Now, we should be able to submit the login form so many times without
        # trouble:

        self.browser.open('http://openid.launchpad.dev/')
        for i in range(limit):
            self.browser.getControl(name='email').value = 'something@example.com'
            self.browser.getControl(name='password').value = 'wrong'
            self.browser.getControl(name='continue').click()


        # But on the next request, it should fail:

        self.browser.getControl(name='email').value = 'something@example.com'
        self.browser.getControl(name='password').value = 'wrong'
        control = self.browser.getControl(name='continue')

        self.assertRaises(HTTPError, control.click)

        # This shouldn't prevent us from logging in as a different user:

        self.browser.open('http://openid.launchpad.dev/')
        self.browser.getControl(name='email').value = 'test@canonical.com'
        self.browser.getControl(name='password').value = 'test'
        self.browser.getControl(name='continue').click()
        self.assertEquals(self.browser.title.strip(), "Sample Person's details")
