from identityprovider.tests.functional.helpers import FunctionalTestCase


class AdminTestCase(FunctionalTestCase):


    fixtures = ['admin', 'test']


    def select_account_state(self, partial_name):
        status_select = self.browser.getControl(name='status')

        for display_value in status_select.displayOptions:
            if partial_name in display_value:
                status_select.displayValue = [display_value]
                return

        self.fail("Account status with name '%s' was not found" % partial_name)


    def test_suspended_account_should_require_a_password_reset(self):
        """Checking regression in #620462"""

        self.admin_login('admin', 'admin007')

        account_url = 'http://launchpad.dev/admin/identityprovider/account/1/'
        self.browser.open(account_url)

        self.assertContains('mark@example.com')

        self.select_account_state("Suspended")
        self.browser.getControl(name='_save').click()

        self.admin_logout()

        # Try to log in back using just suspended account
        self.login('mark@example.com', 'test')
        # This should produce an error
        self.assertContains(
            '<span class="error">Your account has been deactivated</span>')

        self.admin_login('admin', 'admin007')
        self.browser.open(account_url)

        self.select_account_state("Active")
        self.browser.getControl(name='_save').click()

        self.admin_logout()

        self.login('mark@example.com', 'test')

        # If the url is still the same, that means that the log in was
        # unsuccessful.
        self.assertUrl('http://launchpad.dev/\+login')
        self.assertContains(
            '<span class="error">Password didn\'t match.</span>')
