import re
from urllib import urlencode

from identityprovider.tests.functional.helpers import FunctionalTestCase


class BrandedRPTestCase(FunctionalTestCase):

    def test(self):
        # The OpenIDRPConfig table stores information about known RPs. These
        # can be branded.
        self.create_openid_rp_config(
            trust_root='http://launchpad.dev/+openid-consumer',
            displayname='Example Displayname',
            description='Example Description',
            logo='canonical-store-logo.png')

        # First, we need to associate with the OP.
        self.anon_browser.open('http://openid.launchpad.dev/+openid',
                               data=urlencode({
                                   'openid.mode': 'associate',
                                   'openid.assoc_type': 'HMAC-SHA1'}))

        [assoc_handle] = re.findall('assoc_handle:(.*)',
                                    self.anon_browser.contents)

        # Now pull up the authorization screen.
        args = urlencode({
            'openid.mode': 'checkid_setup',
            'openid.identity': 'http://openid.launchpad.dev/+id/mark_oid',
            'openid.assoc_handle': assoc_handle,
            'openid.return_to': 'http://launchpad.dev/+openid-consumer',
        })
        self.anon_browser.open('http://openid.launchpad.dev/+openid?%s' % args)

        # On this screen, we find our branding information.
        self.assertContains('Sign in', self.anon_browser)

        # Then we log in to ensure the branding is still present:
        self.anon_browser.getControl(name='email').value = 'test@canonical.com'
        self.anon_browser.getControl(name='password').value = 'test'
        self.anon_browser.getControl(name='continue').click()

        self.assertContains('Sign in', self.anon_browser)
