# Description: Unittest2 test discovery and real dotted-path named test selection for Django
# Public Clone URL: git://gist.github.com/1450104.git

from django.conf import settings
from django.test.simple import DjangoTestSuiteRunner, reorder_suite
from django.utils.unittest import TestCase
from django.utils.unittest.loader import defaultTestLoader
from django_jenkins import signals
from django_jenkins.runner import CITestSuiteRunner


class DiscoveryDjangoTestSuiteRunner(DjangoTestSuiteRunner):
    def build_suite(self, test_labels, extra_tests=None, **kwargs):
        if test_labels:
            suite = defaultTestLoader.loadTestsFromNames(test_labels)
        else:
            suite = defaultTestLoader.discover(settings.TEST_DISCOVERY_ROOT)

        if extra_tests:
            for test in extra_tests:
                suite.addTest(test)

        return reorder_suite(suite, (TestCase,))


class DiscoveryCITestSuiteRunner(CITestSuiteRunner, DiscoveryDjangoTestSuiteRunner):
    def build_suite(self, test_labels, **kwargs):
        suite = DiscoveryDjangoTestSuiteRunner.build_suite(self, test_labels, **kwargs)
        signals.build_suite.send(sender=self, suite=suite)
        return suite
