import sys
import openid.store.nonce

from datetime import datetime
from time import time
from StringIO import StringIO
from mock import patch

from django.db import connection
from django.contrib.sessions.models import Session
from django.core.management import call_command
from django.test import TestCase

from identityprovider.models import OpenIDNonce

from identityprovider.tests.utils import skipOnSqlite


@skipOnSqlite
class CleanupCommandTestCase(TestCase):
    def tearDown(self):
        cursor = connection.cursor()
        cursor.execute("""TRUNCATE lp_openididentifier, account,
            accountpassword, emailaddress, lp_person, django_session,
            openidnonce, openidassociation CASCADE""")
        connection.connection.commit()

    setUp = tearDown

    def populate(self):
        call_command('populate', accounts=0, sessions=500,
            nonces=500, associations=500, verbosity=0)

        self.assertEqual(500, Session.objects.count())
        self.assertEqual(500, OpenIDNonce.objects.count())

    def test_cleanup(self):
        self.populate()

        mock_stdout = StringIO()
        with patch.object(sys, 'stdout', mock_stdout):
            call_command('cleanup', verbosity=0)
        mock_stdout.seek(0)
        self.assertTrue('No items selected to clean up.' in mock_stdout.read())

    def test_cleanup_all(self):
        self.populate()
        call_command('cleanup', sessions=True, nonces=True, verbosity=0)

        self.assertEqual(0, Session.objects.filter(
            expire_date__lt=datetime.utcnow()).count())
        self.assertEqual(0, OpenIDNonce.objects.filter(timestamp__lt=(
            int(time()) - openid.store.nonce.SKEW)).count())

    def test_cleanup_sessions(self):
        self.populate()
        call_command('cleanup', sessions=True)

        self.assertEqual(0, Session.objects.filter(
            expire_date__lt=datetime.utcnow()).count())
        self.assertEqual(500, OpenIDNonce.objects.count())

    def test_cleanup_nonces(self):
        self.populate()
        call_command('cleanup', nonces=True)

        self.assertEqual(500, Session.objects.count())
        self.assertEqual(0, OpenIDNonce.objects.filter(timestamp__lt=(
            int(time()) - openid.store.nonce.SKEW)).count())
