from datetime import datetime
from django.test import TestCase

from mock import Mock, patch

from identityprovider.decorators import ratelimit


class RatelimitTestCase(TestCase):
    @patch('identityprovider.decorators.datetime')
    def test_keys_to_check_use_utc(self, mock_datetime):
        mock_datetime.utcnow.return_value = datetime.utcnow()
        mock_datetime.side_effect = lambda *args, **kw: datetime(*args, **kw)

        request = Mock()
        limiter = ratelimit()
        with patch.object(limiter, 'key_extra') as mock_key_extra:
            mock_key_extra.return_value = None
            limiter.keys_to_check(request)

        self.assertEqual(mock_datetime.utcnow.called, True)

    @patch('identityprovider.decorators.datetime')
    def test_current_key_use_utc(self, mock_datetime):
        mock_datetime.utcnow.return_value = datetime.utcnow()
        mock_datetime.side_effect = lambda *args, **kw: datetime(*args, **kw)

        request = Mock()
        limiter = ratelimit()
        with patch.object(limiter, 'key_extra') as mock_key_extra:
            mock_key_extra.return_value = None
            limiter.current_key(request)

        self.assertEqual(mock_datetime.utcnow.called, True)
