# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from django.conf import settings
from django.shortcuts import render_to_response
from django.template import RequestContext

import identityprovider.branding
import identityprovider.context_processors
from identityprovider.tests.utils import SSOBaseTestCase


class DummyRequest(object):
    META = {}


class BrandingTestCase(SSOBaseTestCase):

    def setUp(self):
        self.BRAND = settings.BRAND

    def tearDown(self):
        self.set_branding(self.BRAND)

    def set_branding(self, branding):
        # reload settings
        settings.BRAND = branding

        reload(identityprovider.branding)
        reload(identityprovider.context_processors)

    def test_openid(self):
        self.set_branding('Ubuntu')
        r = self.client.get('/+openid')
        self.assertContains(r, "This is the Ubuntu SSO service")
        self.set_branding('Launchpad')
        r = self.client.get('/+openid')
        self.assertContains(r, "This is the Launchpad login service")

    def test_decide(self):
        self.set_branding('Ubuntu')
        req = DummyRequest()
        context = RequestContext(req, {'trust_root': 'foo', })
        response = render_to_response('decide.html', context)
        self.assertContains(response, 'recognised by Ubuntu')

        self.set_branding('Launchpad')
        req = DummyRequest()
        context = RequestContext(req, {'trust_root': 'foo', })
        response = render_to_response('decide.html', context)
        self.assertContains(response, 'recognised by Launchpad')


class StaticPageTestCase(SSOBaseTestCase):

    fixtures = ["test"]

    def setUp(self):
        self.brand = settings.BRAND

    def tearDown(self):
        self.set_brand(self.brand)

    def set_brand(self, brand):
        settings.BRAND = brand
        import identityprovider.urls
        reload(identityprovider.urls)

    def assertAvailable(self, url, text):
        r = self.client.get(url)
        self.assertContains(r, text)

    def assertNotAvailable(self, url):
        r = self.client.get(url)
        self.assertEquals(r.status_code, 404)

    def assertUbuntuOnly(self, url, text):
        self.assertAvailable(url, text)
        self.set_brand('Launchpad')
        self.assertNotAvailable(url)

    def test_faq_page_ubuntu_only(self):
        self.assertUbuntuOnly('/+faq', "Frequently asked questions")

    def test_description_page_ubuntu_only(self):
        self.assertUbuntuOnly('/+description', "What's this?")

    def test_applications_page_ubuntu_only(self):
        self.client.login(username='mark@example.com', password='test')
        self.assertUbuntuOnly('/+applications', 'Applications you use')
