from django.test import TestCase
from mock import patch

from canonical.isd.tests.django import patch_settings

from identityprovider.models.openidmodels import OpenIDRPConfig


class UbuntuLoginTemplateTestCase(TestCase):
    @patch('identityprovider.views.ui._get_rpconfig_from_request')
    def test_rpconfig_with_logo_url(self, mock_get_rpconfig):
        rpconfig = OpenIDRPConfig(
            trust_root='http://localhost/',
            logo='http://localhost/img.png')
        mock_get_rpconfig.return_value = rpconfig

        response = self.client.get('/+login')
        self.assertTemplateUsed(response, 'registration/login.html')
        self.assertContains(response, "<div id='rpconfig_logo'>")
        self.assertContains(response, "<img src='http://localhost/img.png'/>")

    @patch('identityprovider.views.ui._get_rpconfig_from_request')
    def test_rpconfig_without_logo_url(self, mock_get_rpconfig):
        rpconfig = OpenIDRPConfig(
            trust_root='http://localhost/',
            logo='')
        mock_get_rpconfig.return_value = rpconfig

        response = self.client.get('/+login')
        self.assertTemplateUsed(response, 'registration/login.html')
        self.assertNotContains(response, "<div id='rpconfig_logo'>")


class UbuntuBaseTemplateTestCase(TestCase):
    def test_base_template_includes_analytics(self):
        # Analytics code is included if the analytics id is set.
        with patch_settings(GOOGLE_ANALYTICS_ID='foobar'):
            response = self.client.get('/')

        self.assertTemplateUsed(response, 'base.html')
        self.assertContains(
            response, "_gaq.push(['_setAccount', 'foobar']);")

    def test_base_template_not_includes_analytics(self):
        # If the analytics id is not set the analitycs code is not
        # included.
        with patch_settings(GOOGLE_ANALYTICS_ID=None):
            response = self.client.get('/')

        self.assertTemplateUsed(response, 'base.html')
        self.assertNotContains(response, "_gaq.push")
