from django.conf import settings

from identityprovider.models.account import Account
from identityprovider.tests.utils import SSOBaseTestCase


class PreflightViewAccessTestCase(SSOBaseTestCase):

    fixtures = ["test"]
    URL = '/preflight/'

    def test_non_authenticated_users_are_getting_404(self):
        r = self.client.get(self.URL)
        self.assertEquals(r.status_code, 404)

    def test_account_without_person_receives_404(self):
        account = Account.objects.create_account(
            "test", "abcd@example.com", "test")
        self.client.login(username="abcd@example.com", password="test")
        r = self.client.get(self.URL)
        self.assertEquals(r.status_code, 404)
        account.emailaddress_set.all().delete()
        account.delete()

    def test_person_not_in_the_required_team_will_get_404(self):
        self.client.login(username="test@canonical.com", password="test")
        r = self.client.get(self.URL)
        self.assertEquals(r.status_code, 404)

    def test_person_in_ubuntu_team_can_access_the_page(self):
        settings.SSO_VERSIONS_TEAMS = ["ubuntu-team"]
        self.client.login(username="member@canonical.com", password="test")

        r = self.client.get(self.URL)
        self.assertTemplateUsed(r, "preflight/overview.html")
