# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from identityprovider.models import OpenIDRPConfig
from identityprovider.views import utils
from identityprovider.tests.utils import SSOBaseTestCase


class RedirectionURLForTokenTestCase(SSOBaseTestCase):

    def test_if_token_is_none(self):
        url = utils.redirection_url_for_token(None)
        self.assertEquals(url, "/")

    def test_if_token_is_not_none(self):
        token = "ABCDEFGH"
        url = utils.redirection_url_for_token(token)
        self.assertEquals(url, "/ABCDEFGH/")


class GetRPConfigTestCase(SSOBaseTestCase):
    trust_root1 = 'http://launchpad.dev/'
    trust_root2 = 'http://launchpad.dev'

    def test_get_rpconfig_with_trailing_slash(self):
        expected_rpconfig = OpenIDRPConfig.objects.create(
            trust_root=self.trust_root1)

        rpconfig = utils.get_rpconfig(self.trust_root1)
        self.assertEqual(rpconfig, expected_rpconfig)

        rpconfig = utils.get_rpconfig(self.trust_root2)
        self.assertEqual(rpconfig, expected_rpconfig)

    def test_get_rpconfig_without_trailing_slash(self):
        expected_rpconfig = OpenIDRPConfig.objects.create(
            trust_root=self.trust_root2)

        rpconfig = utils.get_rpconfig(self.trust_root1)
        self.assertEqual(rpconfig, expected_rpconfig)

        rpconfig = utils.get_rpconfig(self.trust_root2)
        self.assertEqual(rpconfig, expected_rpconfig)

    def test_get_rpconfig_partial_match_longer(self):
        OpenIDRPConfig.objects.create(trust_root=self.trust_root1)
        rpconfig = utils.get_rpconfig(self.trust_root1 + 'some/extra/url/')
        self.assertEqual(rpconfig, None)

    def test_get_rpconfig_partial_match_shorter(self):
        OpenIDRPConfig.objects.create(
            trust_root=self.trust_root1 + 'some/extra/url/')
        rpconfig = utils.get_rpconfig(self.trust_root1)
        self.assertEqual(rpconfig, None)
