# Copyright 2010, 2012 Canonical Ltd.  This software is licensed under
# the GNU Affero General Public License version 3 (see the file
# LICENSE).

from django.conf.urls.defaults import *
from django.conf import settings

from identityprovider.branding import current_brand
from identityprovider.models import AUTHTOKEN_PATTERN


repls = {
    'token': '(?P<token>[A-Za-z0-9]{16})/',
    'optional_token': '((?P<token>[A-Za-z0-9]{16})/)?',
    'authtoken': '(?P<authtoken>%s)' % AUTHTOKEN_PATTERN,
    'email_address': '(?P<email_address>.+)'
}

urlpatterns = patterns('identityprovider.views.server',
    (r'^((?P<lang>[A-Za-z_]+)/)?\+openid$', 'openid_provider'),
    (r'^\+xrds$', 'xrds'),
    (r'^\+id/(?P<identifier>[A-Za-z0-9\-_]+)$', 'identity_page'),
    (r'^\+id/(?P<identifier>[A-Za-z0-9\-_]+)/\+xrds$', 'xrds_identity_page'),
    (r'^%(token)s\+decide$' % repls, 'decide'),
    (r'^%(token)s\+cancel$' % repls, 'cancel'),
    (r'^%(token)s\+untrusted$' % repls, 'untrusted'),
    (r'\+pre-authorize-rp$', 'pre_authorize'),
)

urlpatterns += patterns('identityprovider.views.ui',
    (r'^%(optional_token)s\+login$' % repls, 'login'),
    (r'^%(optional_token)s\+logout$' % repls, 'logout'),
    # expanded new_account urls these as per comment below to allow for 
    # reverse look up
    (r'^\+new_account$' % repls, 'new_account'),
    (r'^%(token)s\+new_account$' % repls, 'new_account'),
    (r'^%(optional_token)s\+forgot_password$' % repls, 'forgot_password'),
    (r'^%(optional_token)s\+enter_token$' % repls, 'enter_token'),
    (r'^token/%(authtoken)s/$' % repls, 'claim_token'),

    # Django's URL-reversing doesn't like to play with regex
    # alternates '|' or optionals '?'.  Named parameters in such
    # clauses get clobbered to such names as '_0'.  Breaking out the
    # combinations ourselves ensures the names are maintained.
    (r'^token/%(authtoken)s/\+resetpassword/%(email_address)s$' % repls,
     'reset_password'),
    (r'^%(token)stoken/%(authtoken)s/\+resetpassword/%(email_address)s$' \
     % repls, 'reset_password'),
    # {{workflow}}
    (r'^token/%(authtoken)s/\+newaccount/%(email_address)s$' % repls,
     'old_confirm_account'),
    (r'^confirm-account/%(authtoken)s/%(email_address)s$' % repls,
     'confirm_account'),
    (r'^%(token)sconfirm-account/%(authtoken)s/%(email_address)s$' % repls,
     'confirm_account'),
    (r'^token/%(authtoken)s/\+newemail/%(email_address)s$' \
     % repls, 'confirm_email'),
    (r'^%(token)stoken/%(authtoken)s/\+newemail/%(email_address)s$' \
     % repls, 'confirm_email'),

    (r'^\+bad-token', 'bad_token'),
    (r'^\+logout-to-confirm', 'logout_to_confirm'),
    (r'^debug-token/(?P<email>.*\@.*\..*)$', 'token_from_email'),
    (r'^\+deactivated$', 'deactivated'),
    (r'^\+suspended$', 'suspended'),
)

urlpatterns += patterns('identityprovider.views.account',
    (r'^%(optional_token)s\+edit$' % repls, 'index'),
    (r'^\+emails$', 'account_emails'),
    (r'^$', 'index'),
    (r'^\+cookie$', 'cookie'),
    (r'^%(optional_token)s\+index$' % repls, 'index'),
    (r'^%(optional_token)s\+new-email$' % repls, 'new_email'),
    (r'^%(optional_token)s\+verify-email$' % repls, 'verify_email'),
    (r'^%(optional_token)s\+remove-email$' % repls, 'delete_email'),
)

urlpatterns += patterns('identityprovider.views.devices',
    (r'^device-list$', 'device_list'),
    (r'^device-addition$', 'device_addition'),
    (r'^device-verification/(?P<device_id>.+)$', 'device_verification'),
    (r'^device-removal/(?P<device_id>.+)$', 'device_removal'),
    (r'^pad-removal$', 'pad_removal')
)

if settings.BRAND.lower() == 'ubuntu':
    urlpatterns += patterns('identityprovider.views.account',
        (r'^\+applications$', 'applications'),
    )

urlpatterns += patterns('identityprovider.views.i18n',
        (r'^set_language$', 'set_language'),
)

urlpatterns += patterns('identityprovider.views.readonly',
    (r'^readonly$', 'readonly_admin'),
    (r'^readonly/((?P<appserver>[A-Za-z0-9\-_.:]+)/)?(?P<action>enable|disable|set|clear)(/(?P<conn>[A-Za-z0-9\-_.]+))?',
     'readonly_confirm'),
    (r'^readonlydata$', 'readonly_data'),
)

urlpatterns += patterns('django.views.generic.simple',
    (r'^%(token)s$' % repls, 'redirect_to',
        {'url': '/%(token)s/+decide', 'permanent': False}),
    (r'^\+id/(?P<identifier>[A-Za-z0-9\-_]+)/\+index$', 'redirect_to',
        {'url': '/+id/%(identifier)s', 'permanent': False}),
)

if settings.DEBUG:
    urlpatterns += patterns('identityprovider.views.testing',
        (r'^\+openid-consumer', 'openid_consumer'),
        (r'^~(?P<username>[A-Za-z0-9\-_]+)(/(?P<version>[1|2]))?$',
            'delegate_profile'),
        (r'^error$', 'error'),
    )
    urlpatterns += patterns('',
        (r'(favicon.ico)', 'django.views.static.serve',
            {'document_root': settings.SSO_MEDIA_ROOT +
             current_brand.template_dir}),
        (r'^i18n/', include('django.conf.urls.i18n')),
    )

if getattr(settings, 'TESTING', False):
    urlpatterns += patterns('identityprovider.views.consumer',
        (r'^consumer/$', 'startOpenID'),
        (r'^consumer/finish/$', 'finishOpenID'),
        (r'^consumer/xrds/$', 'rpXRDS'),
    )
    urlpatterns += patterns('identityprovider.views.account',
        (r'^%(optional_token)s\+deactivate$' % repls, 'account_deactivate'),
    )
    urlpatterns += patterns('identityprovider.views.testing',
        (r'^dummy$', 'dummy'),
    )

# Keep this at the end, so no important URL will be overridden
if settings.BRAND.lower() == 'ubuntu':
    urlpatterns += patterns('identityprovider.views.ui',
        (r'^\+(?P<page_name>description|faq)$', 'static_page'),
    )

# Lazr.restful backwards compatible api
urlpatterns += patterns('',
    (r'api/1.0/', include('identityprovider.api10.urls')),
)
