# Copyright 2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from saml2idp import exceptions, google_apps, salesforce

from ..auth import internal_authorize

class CanonicalOverrides(object):
    def _determine_subject(self):
        self._subject = self._django_request.user.preferredemail

    def _validate_user(self):
        # Based on identityprovider.auth.internal_authorize.
        user = self._django_request.user
        if internal_authorize(user, 'SAML2_TEAMS'):
            return
        raise exceptions.UserNotAuthorized('User account not enabled for SAML authentication.')

class GoogleAppsProcessor(CanonicalOverrides, google_apps.Processor):
    """
    Google Apps SAML 2.0 Processor, specific to Canonical Identity Provider.
    """

class SalesForceProcessor(CanonicalOverrides, salesforce.Processor):
    """
    SalesForce SAML 2.0 Processor, specific to Canonical Identity Provider.
    """
