# Copyright 2010, 2012 Canonical Ltd.  This software is licensed under
# the GNU Affero General Public License version 3 (see the file
# LICENSE).

from functools import wraps

from django.conf import settings
from django.http import HttpResponseNotAllowed, HttpResponseRedirect

from identityprovider.models import OpenIDRPConfig


class HttpResponseSeeOther(HttpResponseRedirect):
    status_code = 303


def allow_only(*methods):
    def decorator(fn):
        @wraps(fn)
        def wrapper(request, *args, **kwargs):
            if request.method not in methods:
                return HttpResponseNotAllowed(methods)
            return fn(request, *args, **kwargs)
        return wrapper
    return decorator


def redirection_url_for_token(token):
    if token is not None:
        return "/%s/" % token
    else:
        return "/"


def set_session_token_info(session, token):
    """Places information about the token into the session in a
    uniform place.  Specifically, places the token's e-mail address at
    session['token_email'].  Also, if settings.ENABLE_TOKEN_DEBUG is
    true, places the token string at session['token_string']."""
    session['token_email'] = token.email
    if getattr(settings, 'ENABLE_TOKEN_DEBUG', False):
        session['token_string'] = token.token


def get_rpconfig(trust_root):
    alternatives = [trust_root]

    if trust_root.endswith('/'):
        alternatives.append(trust_root[:-1])
    else:
        alternatives.append(trust_root + '/')

    rpconfig = OpenIDRPConfig.objects.filter(trust_root__in=alternatives)
    return rpconfig and rpconfig[0] or None
