# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
from setuptools import setup


# This function was taken from the django project.
# Please see the license file in the third-party/django directory.
def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

data_files = []
packages = []
for dirpath, dirnames, filenames in os.walk('identityprovider'):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'):
            del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([os.path.join('share', dirpath),
                           [os.path.join(dirpath, f) for f in filenames]])

tests_require = [
    'BeautifulSoup==3.1.0.1',
    'coverage==3.4',
    'mock==0.7.2',
    'wsgiref==0.1.2',
    'lazr.restfulclient',
    'wsgi-intercept==0.4',
    'zope.testbrowser==3.5.1',
    'httplib2==0.6.0',
    'django-jenkins',
]
setup(
    name="canonical-identity-provider",
    version="11.07.12",

    author="Canonical ISD Hackers",
    author_email="canonical-isd@lists.launchpad.net",

    license="AGPLv3",

    zip_safe=False,

    packages=packages,
    package_data={
        'identityprovider.webservice': ['site.zcml'],
    },
    data_files=data_files,

    entry_points={
        'console_scripts': 'sso-account-merge = identityprovider.bin.accounts_merge:main'
    },

    install_requires=[
# These are the core requirements, pinned to versions available on
# Lucid.
        'Django==1.3',
        'South==0.6',
        'configglue==1.0.1',
        'django-configglue==0.6.1',
        # need to install it via requirements.txt until a release is made:
        #'django-oauth-backend',
        #'django-openid-auth',
        'django-preflight<0.2',
# PyPi calls upstream's 1.0a "1.0.1" maybe because it doesn't
# understand letters in version strings.  Lucid's package uses "1.0a"
# as upstream does.
        'oauth>=1.0a,<=1.0.1',
        'psycopg2==2.0.13',
        'python-memcached==1.44',
        'python-openid==2.2.4',
        'django-piston',
        # jsonfield is required by Gargoyle; not pegging it here
        # causes bootstrap to fail:
        'django-jsonfield<0.7',
        'gargoyle',
        'pystatsd',
    ],
    tests_require=tests_require,
    extras_require={'test': tests_require},
    dependency_links=[
        'http://initd.org/pub/software/psycopg/PSYCOPG-2-0/',
    ],
)
