#!/usr/bin/env python

from wsgiref.simple_server import make_server
import cgi


otp_status = {
    '0000': 'OK',
    '1111': 'BAD_OTP',
    '2222': 'REPLAYED_OTP',
    '3333': 'BAD_SIGNATURE',
    '4444': 'MISSING_PARAMETER',
    '5555': 'NO_SUCH_CLIENT',
    '6666': 'OPERATION_NOT_ALLOWED',
    '7777': 'BACKEND_ERROR',
    '8888': 'NOT_ENOUGH_ANSWERS',
    '9999': 'REPLAYED_REQUEST',
    # the actual demo keys, so we can demo with real keys
    'cccccccitngh': 'OK', # Demo key 1 prefix
    'cccccccitngv': 'OK', # Demo key 2 prefix
    'cccccccitnhg': 'OK', # Demo key 3 prefix
    # other misc keys
    'cccccccitnhh': 'OK', # Selene's key prefix
}


def app(environ, start_response):
    headers = [('Content-Type', 'text/plain')]
    path = environ['PATH_INFO']
    qs = environ['QUERY_STRING']
    if path == '/':
        start_response('200 OK', headers)
        return ['server is up!']

    if qs:
        args = qs
    else:
        args = path
    params = cgi.parse_qs(args)

    try:
        otp = params['otp'][0]
        nonce = params['nonce'][0]
    except KeyError:
        start_response('500 INTERNAL SERVER ERROR', headers)
        return ['No Soup For You!']

    try:
        dID = otp
        # use only the device's unique ID prefix, if possible
        if 44 <= len(otp) <= 48:
            dID = otp[:-32]
        status = otp_status[dID]
    except KeyError:
        start_response('500 INTERNAL SERVER ERROR', headers)
        return ['Bad Key!']

    start_response('200 OK', headers)
    return [
        'h=kPbo1ld2KhB39Da1RIiGJy5W57w=\n',
        't=2011-05-09T14:45:39Z0595\n',
        'otp=%s\n' % otp,
        'nonce=%s\n' % nonce,
        'sl=25' + '\n',
        'status=%s' % status,
    ]


def start_server(host='', port=8080):
    server = make_server(host, port, app)
    server.serve_forever()


def make_app():
    return app


if __name__ == '__main__':
    start_server()
